@@ -1,10 +1,9 @@
-#![allow(unused_variables)]
-
 use core::fmt::{Display, Formatter};
 
 /// This is the error type used by Postcard
 #[derive(Clone, Debug, Eq, PartialEq)]
 #[cfg_attr(feature = "use-defmt", derive(defmt::Format))]
+#[non_exhaustive]
 pub enum Error {
     /// This is a feature that PostCard will never implement
     WontImplement,
@@ -34,6 +33,8 @@ pub enum Error {
     SerdeSerCustom,
     /// Serde Deserialization Error
     SerdeDeCustom,
+    /// Error while processing `collect_str` during serialization
+    CollectStrError,
 }
 
 impl Display for Error {
@@ -61,6 +62,7 @@ impl Display for Error {
                 DeserializeBadEncoding => "The original data was not well encoded",
                 SerdeSerCustom => "Serde Serialization Error",
                 SerdeDeCustom => "Serde Deserialization Error",
+                CollectStrError => "Error while processing `collect_str` during serialization",
             }
         )
     }
