@@ -1,6 +1,6 @@
-use serde::Serialize;
 use crate::error::{Error, Result};
-use crate::ser::flavors::{Cobs, SerFlavor, Slice};
+use crate::ser::flavors::{Cobs, Flavor, Slice};
+use serde::Serialize;
 
 #[cfg(feature = "heapless")]
 use crate::ser::flavors::HVec;
@@ -8,9 +8,6 @@ use crate::ser::flavors::HVec;
 #[cfg(feature = "heapless")]
 use heapless::Vec;
 
-#[cfg(feature = "use-std")]
-use crate::ser::flavors::StdVec;
-
 #[cfg(feature = "alloc")]
 use crate::ser::flavors::AllocVec;
 
@@ -23,13 +20,11 @@ pub mod flavors;
 pub(crate) mod serializer;
 
 /// Serialize a `T` to the given slice, with the resulting slice containing
-/// data in a serialized then COBS encoded format. The terminating sentinel `0x00` byte is included
-/// in the output buffer.
+/// data in a serialized then COBS encoded format. The terminating sentinel
+/// `0x00` byte is included in the output buffer.
 ///
-/// When successful, this function returns the slices containing:
-///
-/// 1. A slice that contains the serialized and encoded message
-/// 2. A slice that contains the unused portion of the given buffer
+/// When successful, this function returns the slice containing the
+/// serialized and encoded message.
 ///
 /// ## Example
 ///
@@ -63,10 +58,8 @@ where
 /// Serialize a `T` to the given slice, with the resulting slice containing
 /// data in a serialized format.
 ///
-/// When successful, this function returns the slices containing:
-///
-/// 1. A slice that contains the serialized message
-/// 2. A slice that contains the unused portion of the given buffer
+/// When successful, this function returns the slice containing the
+/// serialized message
 ///
 /// ## Example
 ///
@@ -98,7 +91,7 @@ where
 
 /// Serialize a `T` to a `heapless::Vec<u8>`, with the `Vec` containing
 /// data in a serialized then COBS encoded format. The terminating sentinel
-/// `0x00` byte is included in the output `Vec`. Requires the (default) `heapless` feature.
+/// `0x00` byte is included in the output `Vec`.
 ///
 /// ## Example
 ///
@@ -123,6 +116,7 @@ where
 /// assert_eq!(ser.deref(), &[0x02, 0x01, 0x03, 0x20, 0x30, 0x00]);
 /// ```
 #[cfg(feature = "heapless")]
+#[cfg_attr(doc_cfg, doc(cfg(feature = "heapless")))]
 pub fn to_vec_cobs<T, const B: usize>(value: &T) -> Result<Vec<u8, B>>
 where
     T: Serialize + ?Sized,
@@ -131,7 +125,7 @@ where
 }
 
 /// Serialize a `T` to a `heapless::Vec<u8>`, with the `Vec` containing
-/// data in a serialized format. Requires the (default) `heapless` feature.
+/// data in a serialized format.
 ///
 /// ## Example
 ///
@@ -156,6 +150,7 @@ where
 /// assert_eq!(ser.deref(), &[0x01, 0x00, 0x20, 0x30]);
 /// ```
 #[cfg(feature = "heapless")]
+#[cfg_attr(doc_cfg, doc(cfg(feature = "heapless")))]
 pub fn to_vec<T, const B: usize>(value: &T) -> Result<Vec<u8, B>>
 where
     T: Serialize + ?Sized,
@@ -163,7 +158,7 @@ where
     serialize_with_flavor::<T, HVec<B>, Vec<u8, B>>(value, HVec::default())
 }
 
-/// Serialize a `T` to a `std::vec::Vec<u8>`. Requires the `use-std` feature.
+/// Serialize a `T` to a `std::vec::Vec<u8>`.
 ///
 /// ## Example
 ///
@@ -177,14 +172,16 @@ where
 /// assert_eq!(ser.as_slice(), &[0x03, b'H', b'i', b'!']);
 /// ```
 #[cfg(feature = "use-std")]
+#[cfg_attr(doc_cfg, doc(cfg(feature = "use-std")))]
+#[inline]
 pub fn to_stdvec<T>(value: &T) -> Result<std::vec::Vec<u8>>
 where
     T: Serialize + ?Sized,
 {
-    serialize_with_flavor::<T, StdVec, std::vec::Vec<u8>>(value, StdVec(std::vec::Vec::new()))
+    to_allocvec(value)
 }
 
-/// Serialize and COBS encode a `T` to a `std::vec::Vec<u8>`. Requires the `use-std` feature.
+/// Serialize and COBS encode a `T` to a `std::vec::Vec<u8>`.
 ///
 /// The terminating sentinel `0x00` byte is included in the output.
 ///
@@ -200,17 +197,16 @@ where
 /// assert_eq!(ser.as_slice(), &[0x05, 0x03, b'H', b'i', b'!', 0x00]);
 /// ```
 #[cfg(feature = "use-std")]
+#[cfg_attr(doc_cfg, doc(cfg(feature = "use-std")))]
+#[inline]
 pub fn to_stdvec_cobs<T>(value: &T) -> Result<std::vec::Vec<u8>>
 where
     T: Serialize + ?Sized,
 {
-    serialize_with_flavor::<T, Cobs<StdVec>, std::vec::Vec<u8>>(
-        value,
-        Cobs::try_new(StdVec(std::vec::Vec::new()))?,
-    )
+    to_allocvec_cobs(value)
 }
 
-/// Serialize a `T` to an `alloc::vec::Vec<u8>`. Requires the `alloc` feature.
+/// Serialize a `T` to an `alloc::vec::Vec<u8>`.
 ///
 /// ## Example
 ///
@@ -224,14 +220,15 @@ where
 /// assert_eq!(ser.as_slice(), &[0x03, b'H', b'i', b'!']);
 /// ```
 #[cfg(feature = "alloc")]
+#[cfg_attr(doc_cfg, doc(cfg(feature = "alloc")))]
 pub fn to_allocvec<T>(value: &T) -> Result<alloc::vec::Vec<u8>>
 where
     T: Serialize + ?Sized,
 {
-    serialize_with_flavor::<T, AllocVec, alloc::vec::Vec<u8>>(value, AllocVec(alloc::vec::Vec::new()))
+    serialize_with_flavor::<T, AllocVec, alloc::vec::Vec<u8>>(value, AllocVec::new())
 }
 
-/// Serialize and COBS encode a `T` to an `alloc::vec::Vec<u8>`. Requires the `alloc` feature.
+/// Serialize and COBS encode a `T` to an `alloc::vec::Vec<u8>`.
 ///
 /// The terminating sentinel `0x00` byte is included in the output.
 ///
@@ -247,20 +244,155 @@ where
 /// assert_eq!(ser.as_slice(), &[0x05, 0x03, b'H', b'i', b'!', 0x00]);
 /// ```
 #[cfg(feature = "alloc")]
+#[cfg_attr(doc_cfg, doc(cfg(feature = "alloc")))]
 pub fn to_allocvec_cobs<T>(value: &T) -> Result<alloc::vec::Vec<u8>>
 where
     T: Serialize + ?Sized,
 {
     serialize_with_flavor::<T, Cobs<AllocVec>, alloc::vec::Vec<u8>>(
         value,
-        Cobs::try_new(AllocVec(alloc::vec::Vec::new()))?,
+        Cobs::try_new(AllocVec::new())?,
     )
 }
 
+/// Conveniently serialize a `T` to the given slice, with the resulting slice containing
+/// data followed by a 32-bit CRC. The CRC bytes are included in the output buffer.
+///
+/// When successful, this function returns the slice containing the
+/// serialized and encoded message.
+///
+/// ## Example
+///
+/// ```rust
+/// use crc::{Crc, CRC_32_ISCSI};
+///
+/// let mut buf = [0; 9];
+///
+/// let data: &[u8] = &[0x01, 0x00, 0x20, 0x30];
+/// let crc = Crc::<u32>::new(&CRC_32_ISCSI);
+/// let used = postcard::to_slice_crc32(data, &mut buf, crc.digest()).unwrap();
+/// assert_eq!(used, &[0x04, 0x01, 0x00, 0x20, 0x30, 0x8E, 0xC8, 0x1A, 0x37]);
+/// ```
+///
+/// See the `ser_flavors::crc` module for the complete set of functions.
+#[cfg(feature = "use-crc")]
+#[cfg_attr(doc_cfg, doc(cfg(feature = "use-crc")))]
+#[inline]
+pub fn to_slice_crc32<'a, 'b, T>(
+    value: &'b T,
+    buf: &'a mut [u8],
+    digest: crc::Digest<'a, u32>,
+) -> Result<&'a mut [u8]>
+where
+    T: Serialize + ?Sized,
+{
+    flavors::crc::to_slice_u32(value, buf, digest)
+}
+
+/// Conveniently serialize a `T` to a `heapless::Vec<u8>`, with the `Vec` containing
+/// data followed by a 32-bit  CRC. The CRC bytes are included in the output `Vec`.
+///
+/// ## Example
+///
+/// ```rust
+/// use crc::{Crc, CRC_32_ISCSI};
+/// use heapless::Vec;
+/// use core::ops::Deref;
+///
+/// // NOTE: postcard handles `&[u8]` and `&[u8; N]` differently.
+/// let data: &[u8] = &[0x01u8, 0x00, 0x20, 0x30];
+/// let crc = Crc::<u32>::new(&CRC_32_ISCSI);
+/// let ser: Vec<u8, 32> = postcard::to_vec_crc32(data, crc.digest()).unwrap();
+/// assert_eq!(ser.deref(), &[0x04, 0x01, 0x00, 0x20, 0x30, 0x8E, 0xC8, 0x1A, 0x37]);
+///
+/// let data: &[u8; 4] = &[0x01u8, 0x00, 0x20, 0x30];
+/// let ser: Vec<u8, 32> = postcard::to_vec_crc32(data, crc.digest()).unwrap();
+/// assert_eq!(ser.deref(), &[0x01, 0x00, 0x20, 0x30, 0xCC, 0x4B, 0x4A, 0xDA]);
+/// ```
+///
+/// See the `ser_flavors::crc` module for the complete set of functions.
+#[cfg(all(feature = "use-crc", feature = "heapless"))]
+#[cfg_attr(doc_cfg, doc(cfg(all(feature = "use-crc", feature = "heapless"))))]
+#[inline]
+pub fn to_vec_crc32<'a, T, const B: usize>(
+    value: &T,
+    digest: crc::Digest<'a, u32>,
+) -> Result<heapless::Vec<u8, B>>
+where
+    T: Serialize + ?Sized,
+{
+    flavors::crc::to_vec_u32(value, digest)
+}
+
+/// Conveniently serialize a `T` to a `heapless::Vec<u8>`, with the `Vec` containing
+/// data followed by a 32-bit  CRC. The CRC bytes are included in the output `Vec`.
+///
+/// ## Example
+///
+/// ```rust
+/// use crc::{Crc, CRC_32_ISCSI};
+/// use core::ops::Deref;
+///
+/// // NOTE: postcard handles `&[u8]` and `&[u8; N]` differently.
+/// let data: &[u8] = &[0x01u8, 0x00, 0x20, 0x30];
+/// let crc = Crc::<u32>::new(&CRC_32_ISCSI);
+/// let ser: Vec<u8> = postcard::to_stdvec_crc32(data, crc.digest()).unwrap();
+/// assert_eq!(ser.deref(), &[0x04, 0x01, 0x00, 0x20, 0x30, 0x8E, 0xC8, 0x1A, 0x37]);
+///
+/// let data: &[u8; 4] = &[0x01u8, 0x00, 0x20, 0x30];
+/// let ser: Vec<u8> = postcard::to_stdvec_crc32(data, crc.digest()).unwrap();
+/// assert_eq!(ser.deref(), &[0x01, 0x00, 0x20, 0x30, 0xCC, 0x4B, 0x4A, 0xDA]);
+/// ```
+///
+/// See the `ser_flavors::crc` module for the complete set of functions.
+#[cfg(all(feature = "use-crc", feature = "use-std"))]
+#[cfg_attr(doc_cfg, doc(cfg(all(feature = "use-crc", feature = "use-std"))))]
+#[inline]
+pub fn to_stdvec_crc32<'a, T>(value: &T, digest: crc::Digest<'a, u32>) -> Result<std::vec::Vec<u8>>
+where
+    T: Serialize + ?Sized,
+{
+    flavors::crc::to_allocvec_u32(value, digest)
+}
+
+/// Conveniently serialize a `T` to a `heapless::Vec<u8>`, with the `Vec` containing
+/// data followed by a 32-bit  CRC. The CRC bytes are included in the output `Vec`.
+///
+/// ## Example
+///
+/// ```rust
+/// use crc::{Crc, CRC_32_ISCSI};
+/// use core::ops::Deref;
+///
+/// // NOTE: postcard handles `&[u8]` and `&[u8; N]` differently.
+/// let data: &[u8] = &[0x01u8, 0x00, 0x20, 0x30];
+/// let crc = Crc::<u32>::new(&CRC_32_ISCSI);
+/// let ser: Vec<u8> = postcard::to_allocvec_crc32(data, crc.digest()).unwrap();
+/// assert_eq!(ser.deref(), &[0x04, 0x01, 0x00, 0x20, 0x30, 0x8E, 0xC8, 0x1A, 0x37]);
+///
+/// let data: &[u8; 4] = &[0x01u8, 0x00, 0x20, 0x30];
+/// let ser: Vec<u8> = postcard::to_allocvec_crc32(data, crc.digest()).unwrap();
+/// assert_eq!(ser.deref(), &[0x01, 0x00, 0x20, 0x30, 0xCC, 0x4B, 0x4A, 0xDA]);
+/// ```
+///
+/// See the `ser_flavors::crc` module for the complete set of functions.
+#[cfg(all(feature = "use-crc", feature = "alloc"))]
+#[cfg_attr(doc_cfg, doc(cfg(all(feature = "use-crc", feature = "alloc"))))]
+#[inline]
+pub fn to_allocvec_crc32<'a, T>(
+    value: &T,
+    digest: crc::Digest<'a, u32>,
+) -> Result<alloc::vec::Vec<u8>>
+where
+    T: Serialize + ?Sized,
+{
+    flavors::crc::to_allocvec_u32(value, digest)
+}
+
 /// `serialize_with_flavor()` has three generic parameters, `T, F, O`.
 ///
 /// * `T`: This is the type that is being serialized
-/// * `F`: This is the Flavor (combination) that is used during serialization
+/// * `S`: This is the Storage that is used during serialization
 /// * `O`: This is the resulting storage type that is returned containing the serialized data
 ///
 /// For more information about how Flavors work, please see the
@@ -269,7 +401,7 @@ where
 /// ```rust
 /// use postcard::{
 ///     serialize_with_flavor,
-///     flavors::{Cobs, Slice},
+///     ser_flavors::{Cobs, Slice},
 /// };
 ///
 /// let mut buf = [0u8; 32];
@@ -283,62 +415,92 @@ where
 ///
 /// assert_eq!(res, &[0x03, 0x04, 0x01, 0x03, 0x20, 0x30, 0x00]);
 /// ```
-pub fn serialize_with_flavor<T, F, O>(value: &T, flavor: F) -> Result<O>
+pub fn serialize_with_flavor<T, S, O>(value: &T, storage: S) -> Result<O>
 where
     T: Serialize + ?Sized,
-    F: SerFlavor<Output = O>,
+    S: Flavor<Output = O>,
 {
-    let mut serializer = Serializer { output: flavor };
+    let mut serializer = Serializer { output: storage };
     value.serialize(&mut serializer)?;
     serializer
         .output
-        .release()
+        .finalize()
         .map_err(|_| Error::SerializeBufferFull)
 }
 
+/// Compute the size of the postcard serialization of `T`.
+pub fn serialized_size<T>(value: &T) -> Result<usize>
+where
+    T: Serialize + ?Sized,
+{
+    serialize_with_flavor::<T, flavors::Size, usize>(value, flavors::Size::default())
+}
+
 #[cfg(feature = "heapless")]
 #[cfg(test)]
 mod test {
     use super::*;
-    use crate::varint::VarintUsize;
+    use crate::max_size::MaxSize;
+    use crate::varint::{varint_max, varint_usize};
     use core::fmt::Write;
     use core::ops::{Deref, DerefMut};
-    use heapless::{String, FnvIndexMap};
+    use heapless::{FnvIndexMap, String};
     use serde::Deserialize;
 
     #[test]
     fn ser_u8() {
         let output: Vec<u8, 1> = to_vec(&0x05u8).unwrap();
         assert!(&[5] == output.deref());
+        assert!(output.len() == serialized_size(&0x05u8).unwrap());
+        assert!(output.len() <= Vec::<u8, 1>::POSTCARD_MAX_SIZE);
     }
 
     #[test]
     fn ser_u16() {
-        let output: Vec<u8, 2> = to_vec(&0xA5C7u16).unwrap();
-        assert!(&[0xC7, 0xA5] == output.deref());
+        const SZ: usize = varint_max::<u16>();
+        let output: Vec<u8, SZ> = to_vec(&0xA5C7u16).unwrap();
+        assert_eq!(&[0xC7, 0xCB, 0x02], output.deref());
+        assert!(output.len() == serialized_size(&0xA5C7u16).unwrap());
+        assert!(output.len() <= Vec::<u8, SZ>::POSTCARD_MAX_SIZE);
     }
 
     #[test]
     fn ser_u32() {
-        let output: Vec<u8, 4> = to_vec(&0xCDAB3412u32).unwrap();
-        assert!(&[0x12, 0x34, 0xAB, 0xCD] == output.deref());
+        const SZ: usize = varint_max::<u32>();
+        let output: Vec<u8, SZ> = to_vec(&0xCDAB3412u32).unwrap();
+        assert_eq!(&[0x92, 0xE8, 0xAC, 0xED, 0x0C], output.deref());
+        assert!(output.len() == serialized_size(&0xCDAB3412u32).unwrap());
+        assert!(output.len() <= Vec::<u8, SZ>::POSTCARD_MAX_SIZE);
     }
 
     #[test]
     fn ser_u64() {
-        let output: Vec<u8, 8> = to_vec(&0x1234_5678_90AB_CDEFu64).unwrap();
-        assert!(&[0xEF, 0xCD, 0xAB, 0x90, 0x78, 0x56, 0x34, 0x12] == output.deref());
+        const SZ: usize = varint_max::<u64>();
+        let output: Vec<u8, SZ> = to_vec(&0x1234_5678_90AB_CDEFu64).unwrap();
+        assert_eq!(
+            &[0xEF, 0x9B, 0xAF, 0x85, 0x89, 0xCF, 0x95, 0x9A, 0x12],
+            output.deref()
+        );
+        assert!(output.len() == serialized_size(&0x1234_5678_90AB_CDEFu64).unwrap());
+        assert!(output.len() <= Vec::<u8, SZ>::POSTCARD_MAX_SIZE);
     }
 
     #[test]
     fn ser_u128() {
-        let output: Vec<u8, 16> = to_vec(&0x1234_5678_90AB_CDEF_1234_5678_90AB_CDEFu128).unwrap();
-        assert!(
+        const SZ: usize = varint_max::<u128>();
+        let output: Vec<u8, SZ> = to_vec(&0x1234_5678_90AB_CDEF_1234_5678_90AB_CDEFu128).unwrap();
+        assert_eq!(
             &[
-                0xEF, 0xCD, 0xAB, 0x90, 0x78, 0x56, 0x34, 0x12,
-                0xEF, 0xCD, 0xAB, 0x90, 0x78, 0x56, 0x34, 0x12
-            ] == output.deref()
+                0xEF, 0x9B, 0xAF, 0x85, 0x89, 0xCF, 0x95, 0x9A, 0x92, 0xDE, 0xB7, 0xDE, 0x8A, 0x92,
+                0x9E, 0xAB, 0xB4, 0x24,
+            ],
+            output.deref()
         );
+        assert!(
+            output.len()
+                == serialized_size(&0x1234_5678_90AB_CDEF_1234_5678_90AB_CDEFu128).unwrap()
+        );
+        assert!(output.len() <= Vec::<u8, SZ>::POSTCARD_MAX_SIZE);
     }
 
     #[derive(Serialize)]
@@ -350,27 +512,38 @@ mod test {
         tt: u32,
     }
 
+    impl MaxSize for BasicU8S {
+        const POSTCARD_MAX_SIZE: usize = {
+            u16::POSTCARD_MAX_SIZE
+                + u8::POSTCARD_MAX_SIZE
+                + u128::POSTCARD_MAX_SIZE
+                + u64::POSTCARD_MAX_SIZE
+                + u32::POSTCARD_MAX_SIZE
+        };
+    }
+
     #[test]
     fn ser_struct_unsigned() {
-        let output: Vec<u8, 31> = to_vec(&BasicU8S {
+        const SZ: usize = BasicU8S::POSTCARD_MAX_SIZE;
+        let input = BasicU8S {
             st: 0xABCD,
             ei: 0xFE,
             ote: 0x1234_4321_ABCD_DCBA_1234_4321_ABCD_DCBA,
             sf: 0x1234_4321_ABCD_DCBA,
             tt: 0xACAC_ACAC,
-        })
-        .unwrap();
+        };
+        let output: Vec<u8, SZ> = to_vec(&input).unwrap();
 
-        assert!(
+        assert_eq!(
             &[
-                0xCD, 0xAB,
-                0xFE,
-                0xBA, 0xDC, 0xCD, 0xAB, 0x21, 0x43, 0x34, 0x12,
-                0xBA, 0xDC, 0xCD, 0xAB, 0x21, 0x43, 0x34, 0x12,
-                0xBA, 0xDC, 0xCD, 0xAB, 0x21, 0x43, 0x34, 0x12,
-                0xAC, 0xAC, 0xAC, 0xAC
-            ] == output.deref()
+                0xCD, 0xD7, 0x02, 0xFE, 0xBA, 0xB9, 0xB7, 0xDE, 0x9A, 0xE4, 0x90, 0x9A, 0x92, 0xF4,
+                0xF2, 0xEE, 0xBC, 0xB5, 0xC8, 0xA1, 0xB4, 0x24, 0xBA, 0xB9, 0xB7, 0xDE, 0x9A, 0xE4,
+                0x90, 0x9A, 0x12, 0xAC, 0xD9, 0xB2, 0xE5, 0x0A
+            ],
+            output.deref()
         );
+        assert!(output.len() == serialized_size(&input).unwrap());
+        assert!(output.len() <= BasicU8S::POSTCARD_MAX_SIZE);
     }
 
     #[test]
@@ -381,6 +554,7 @@ mod test {
             &[0x08, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08],
             output.deref()
         );
+        assert!(output.len() == serialized_size(&input).unwrap());
 
         let mut input: Vec<u8, 1024> = Vec::new();
         for i in 0..1024 {
@@ -401,6 +575,7 @@ mod test {
         let output: Vec<u8, 17> = to_vec(input).unwrap();
         assert_eq!(0x10, output.deref()[0]);
         assert_eq!(input.as_bytes(), &output.deref()[1..]);
+        assert!(output.len() == serialized_size(&input).unwrap());
 
         let mut input: String<1024> = String::new();
         for _ in 0..256 {
@@ -408,6 +583,7 @@ mod test {
         }
         let output: Vec<u8, 2048> = to_vec(input.deref()).unwrap();
         assert_eq!(&[0x80, 0x08], &output.deref()[..2]);
+        assert!(String::<1024>::POSTCARD_MAX_SIZE <= output.len());
 
         assert_eq!(output.len(), 1026);
         for ch in output.deref()[2..].chunks(4) {
@@ -417,21 +593,23 @@ mod test {
 
     #[test]
     fn usize_varint_encode() {
-        let mut buf = VarintUsize::new_buf();
-        let res = VarintUsize(1).to_buf(&mut buf);
+        let mut buf = [0; varint_max::<usize>()];
+        let res = varint_usize(1, &mut buf);
 
         assert!(&[1] == res);
 
-        let res = VarintUsize(usize::max_value()).to_buf(&mut buf);
+        let res = varint_usize(usize::MAX, &mut buf);
 
-        // AJM TODO
-        if VarintUsize::varint_usize_max() == 5 {
+        //
+        if varint_max::<usize>() == varint_max::<u32>() {
             assert_eq!(&[0xFF, 0xFF, 0xFF, 0xFF, 0x0F], res);
-        } else {
+        } else if varint_max::<usize>() == varint_max::<u64>() {
             assert_eq!(
                 &[0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x01],
                 res
             );
+        } else {
+            panic!("Update this test for 16/128 bit targets!");
         }
     }
 
@@ -461,46 +639,60 @@ mod test {
 
     #[test]
     fn enums() {
-        let output: Vec<u8, 1> = to_vec(&BasicEnum::Bim).unwrap();
+        let input = BasicEnum::Bim;
+        let output: Vec<u8, 1> = to_vec(&input).unwrap();
         assert_eq!(&[0x01], output.deref());
+        assert!(output.len() == serialized_size(&input).unwrap());
 
-        let output: Vec<u8, 9> = to_vec(&DataEnum::Bim(u64::max_value())).unwrap();
+        let input = DataEnum::Bim(u64::max_value());
+        let output: Vec<u8, { 1 + varint_max::<u64>() }> = to_vec(&input).unwrap();
         assert_eq!(
-            &[0x01, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF],
+            &[0x01, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0x01],
             output.deref()
         );
+        assert!(output.len() == serialized_size(&input).unwrap());
 
-        let output: Vec<u8, 3> = to_vec(&DataEnum::Bib(u16::max_value())).unwrap();
-        assert_eq!(&[0x00, 0xFF, 0xFF], output.deref());
+        let input = DataEnum::Bib(u16::max_value());
+        let output: Vec<u8, { 1 + varint_max::<u16>() }> = to_vec(&input).unwrap();
+        assert_eq!(&[0x00, 0xFF, 0xFF, 0x03], output.deref());
+        assert!(output.len() == serialized_size(&input).unwrap());
 
-        let output: Vec<u8, 2> = to_vec(&DataEnum::Bap(u8::max_value())).unwrap();
+        let input = DataEnum::Bap(u8::max_value());
+        let output: Vec<u8, 2> = to_vec(&input).unwrap();
         assert_eq!(&[0x02, 0xFF], output.deref());
+        assert!(output.len() == serialized_size(&input).unwrap());
 
-        let output: Vec<u8, 8> = to_vec(&DataEnum::Kim(EnumStruct {
+        let input = DataEnum::Kim(EnumStruct {
             eight: 0xF0,
             sixt: 0xACAC,
-        }))
-        .unwrap();
-        assert_eq!(&[0x03, 0xF0, 0xAC, 0xAC,], output.deref());
+        });
+        let output: Vec<u8, 8> = to_vec(&input).unwrap();
+        assert_eq!(&[0x03, 0xF0, 0xAC, 0xD9, 0x02], output.deref());
+        assert!(output.len() == serialized_size(&input).unwrap());
 
-        let output: Vec<u8, 8> = to_vec(&DataEnum::Chi {
+        let input = DataEnum::Chi {
             a: 0x0F,
             b: 0xC7C7C7C7,
-        })
-        .unwrap();
-        assert_eq!(&[0x04, 0x0F, 0xC7, 0xC7, 0xC7, 0xC7], output.deref());
+        };
+        let output: Vec<u8, 8> = to_vec(&input).unwrap();
+        assert_eq!(&[0x04, 0x0F, 0xC7, 0x8F, 0x9F, 0xBE, 0x0C], output.deref());
+        assert!(output.len() == serialized_size(&input).unwrap());
 
-        let output: Vec<u8, 8> = to_vec(&DataEnum::Sho(0x6969, 0x07)).unwrap();
-        assert_eq!(&[0x05, 0x69, 0x69, 0x07], output.deref());
+        let input = DataEnum::Sho(0x6969, 0x07);
+        let output: Vec<u8, 8> = to_vec(&input).unwrap();
+        assert_eq!(&[0x05, 0xE9, 0xD2, 0x01, 0x07], output.deref());
+        assert!(output.len() == serialized_size(&input).unwrap());
     }
 
     #[test]
     fn tuples() {
-        let output: Vec<u8, 128> = to_vec(&(1u8, 10u32, "Hello!")).unwrap();
+        let input = (1u8, 10u32, "Hello!");
+        let output: Vec<u8, 128> = to_vec(&input).unwrap();
         assert_eq!(
-            &[1u8, 0x0A, 0x00, 0x00, 0x00, 0x06, b'H', b'e', b'l', b'l', b'o', b'!'],
+            &[1u8, 0x0A, 0x06, b'H', b'e', b'l', b'l', b'o', b'!'],
             output.deref()
-        )
+        );
+        assert!(output.len() == serialized_size(&input).unwrap());
     }
 
     #[test]
@@ -508,10 +700,13 @@ mod test {
         let x: &[u8; 32] = &[0u8; 32];
         let output: Vec<u8, 128> = to_vec(x).unwrap();
         assert_eq!(output.len(), 32);
+        assert!(output.len() == serialized_size(&x).unwrap());
+        assert!(<[u8; 32] as MaxSize>::POSTCARD_MAX_SIZE <= output.len());
 
         let x: &[u8] = &[0u8; 32];
         let output: Vec<u8, 128> = to_vec(x).unwrap();
         assert_eq!(output.len(), 33);
+        assert!(output.len() == serialized_size(&x).unwrap());
     }
 
     #[derive(Serialize)]
@@ -522,11 +717,15 @@ mod test {
 
     #[test]
     fn structs() {
-        let output: Vec<u8, 4> = to_vec(&NewTypeStruct(5)).unwrap();
-        assert_eq!(&[0x05, 0x00, 0x00, 0x00], output.deref());
+        let input = NewTypeStruct(5);
+        let output: Vec<u8, 1> = to_vec(&input).unwrap();
+        assert_eq!(&[0x05], output.deref());
+        assert!(output.len() == serialized_size(&input).unwrap());
 
-        let output: Vec<u8, 3> = to_vec(&TupleStruct((0xA0, 0x1234))).unwrap();
-        assert_eq!(&[0xA0, 0x34, 0x12], output.deref());
+        let input = TupleStruct((0xA0, 0x1234));
+        let output: Vec<u8, 3> = to_vec(&input).unwrap();
+        assert_eq!(&[0xA0, 0xB4, 0x24], output.deref());
+        assert!(output.len() == serialized_size(&input).unwrap());
     }
 
     #[derive(Serialize, Deserialize, Eq, PartialEq, Debug)]
@@ -539,22 +738,24 @@ mod test {
     fn ref_struct() {
         let message = "hElLo";
         let bytes = [0x01, 0x10, 0x02, 0x20];
-        let output: Vec<u8, 11> = to_vec(&RefStruct {
+        let input = RefStruct {
             bytes: &bytes,
             str_s: message,
-        })
-        .unwrap();
+        };
+        let output: Vec<u8, 11> = to_vec(&input).unwrap();
 
         assert_eq!(
             &[0x04, 0x01, 0x10, 0x02, 0x20, 0x05, b'h', b'E', b'l', b'L', b'o',],
             output.deref()
         );
+        assert!(output.len() == serialized_size(&input).unwrap());
     }
 
     #[test]
     fn unit() {
         let output: Vec<u8, 1> = to_vec(&()).unwrap();
         assert_eq!(output.len(), 0);
+        assert!(output.len() == serialized_size(&()).unwrap());
     }
 
     #[test]
@@ -563,11 +764,13 @@ mod test {
         input.extend_from_slice(&[0x01, 0x02, 0x03, 0x04]).unwrap();
         let output: Vec<u8, 5> = to_vec(&input).unwrap();
         assert_eq!(&[0x04, 0x01, 0x02, 0x03, 0x04], output.deref());
+        assert!(output.len() == serialized_size(&input).unwrap());
 
         let mut input: String<8> = String::new();
         write!(&mut input, "helLO!").unwrap();
         let output: Vec<u8, 7> = to_vec(&input).unwrap();
         assert_eq!(&[0x06, b'h', b'e', b'l', b'L', b'O', b'!'], output.deref());
+        assert!(output.len() == serialized_size(&input).unwrap());
 
         let mut input: FnvIndexMap<u8, u8, 4> = FnvIndexMap::new();
         input.insert(0x01, 0x05).unwrap();
@@ -575,7 +778,11 @@ mod test {
         input.insert(0x03, 0x07).unwrap();
         input.insert(0x04, 0x08).unwrap();
         let output: Vec<u8, 100> = to_vec(&input).unwrap();
-        assert_eq!(&[0x04, 0x01, 0x05, 0x02, 0x06, 0x03, 0x07, 0x04, 0x08], output.deref());
+        assert_eq!(
+            &[0x04, 0x01, 0x05, 0x02, 0x06, 0x03, 0x07, 0x04, 0x08],
+            output.deref()
+        );
+        assert!(output.len() == serialized_size(&input).unwrap());
     }
 
     #[test]
